/*
  WOWGLUE.C - 1994 James M. Finnegan
*/
#include <windows.h>
#include <dos.h>
#include <stdarg.h>
#include "wowglue.h"

// Capability bits (returned by WOWGetCaps32())
DWORD dwCapBits=0;

// Function pointers to stuff in Kernel (16 bit)
HINSTANCE32 (WINAPI *LoadLibraryEx32W) (LPSTR, HFILE32, DWORD);
BOOL        (WINAPI *FreeLibrary32W)   (HINSTANCE32);
FARPROC     (WINAPI *GetProcAddress32W)(HINSTANCE32, LPCSTR);
DWORD       (WINAPI *GetVDMPointer32W) (LPVOID, UINT);
DWORD       (WINAPI *CallProc32W)      (FARPROC, DWORD, DWORD);


/*
Wow! a LibMain that does something!
*/
int WINAPI LibMain(HANDLE hInstance, WORD wDataSeg, WORD wHeapSize,
                       LPSTR lpszCmdLine)
{
    HMODULE hKernel;
    
    
    // Yawn!
    if(wHeapSize > 0)
        UnlockData(0);

    // Initialize capability bits for supplied functions
    dwCapBits =WOW_VDMPTR16;
    dwCapBits|=WOW_HWND32;
    
    // Get Kernel handle (we're already running in 16 bit mode so its 
    // gotta be loaded, right?!
    hKernel=GetModuleHandle("KERNEL");
    
    // Dynamically link to the functions we want, setting the capability
    // bits as needed...
    
    if((FARPROC)LoadLibraryEx32W =GetProcAddress(hKernel,"LoadLibraryEx32W"))
        dwCapBits |= WOW_LOADLIBRARY;

    if((FARPROC)FreeLibrary32W   =GetProcAddress(hKernel,"FreeLibrary32W"))
        dwCapBits |= WOW_FREELIBRARY;

    if((FARPROC)GetProcAddress32W=GetProcAddress(hKernel,"GetProcAddress32W"))
        dwCapBits |= WOW_GETPROCADDRESS;

    if((FARPROC)GetVDMPointer32W =GetProcAddress(hKernel,"GetVDMPointer32W"))
        dwCapBits |= WOW_VDMPTR32;

    if((FARPROC)CallProc32W      =GetProcAddress(hKernel,"CallProc32W"))
        dwCapBits |= WOW_CALLPROC;
                               
    return 1;
}


/*
WOWGetCaps32 - This function returns the capability DWORD that was
                  assembled during DLL initialization
*/
DWORD __export WINAPI WOWGetCaps32(VOID)
{
    return dwCapBits;
}


/*
WOWCreateVDMPointer16 - This function allocates and assigns a selector based
                        on the 32 bit linear address passed in.

Returns - The 16 bit selector created.
*/
UINT __export WINAPI WOWCreateVDMPointer16(DWORD dwLinBase, DWORD dwLimit)
{
    UINT  uSelector,
          uDSSel;
                              
                              
    if(!(dwCapBits & WOW_VDMPTR16))
        return NULL;
        
    // Grab our DS (for access rights)
    __asm mov uDSSel, ds
    
    // Allocate a new selector
    if(uSelector=AllocSelector(uDSSel))
    {
        // Assign its linear base address and limit
        SetSelectorBase(uSelector, dwLinBase);
        SetSelectorLimit(uSelector, dwLimit);
    }
    
    return uSelector;
}


/*
WOWDeleteVDMPointer16 - This function deletes the selector that was created
                        by WOWCreateVDMPointer16.

Returns - 0 if the selector is deleted.
*/
UINT __export WINAPI WOWDeleteVDMPointer16(UINT uSelector)
{
    if(!(dwCapBits & WOW_VDMPTR16))
        return NULL;
        
    return FreeSelector(uSelector); 
}


/*
WOWHwndToHwnd32 - This function converts a 16-bit HWND to a 32-bit
HWND for calls to Win32.

Returns - The 32 bit HWND.
*/
HWND32 __export WINAPI WOWHwndToHwnd32(HWND hWnd)
{
    if(!(dwCapBits & WOW_HWND32))
        return NULL;
    
    // OR mask the upper 16 bits
    return hWnd | 0xffff0000;
}


/*
WOWLoadLibraryEx32 - This function calls thru to the Kernel call.

Returns - 32 bit instance handle of the loaded file.
*/
HINSTANCE32 __export WINAPI WOWLoadLibraryEx32(LPSTR lpszFile, HFILE32 hFile, 
                                                            DWORD dwFlags)
{
    if(!(dwCapBits & WOW_LOADLIBRARY))
        return NULL;
    
    return LoadLibraryEx32W(lpszFile, hFile, dwFlags);
}


/*
WOWFreeLibrary32 - This function calls thru to the Kernel call.

Returns - True if successful.
*/
BOOL __export WINAPI WOWFreeLibrary32(HINSTANCE32 hInst32)
{
    if(!(dwCapBits & WOW_FREELIBRARY))
        return NULL;
    
    return FreeLibrary32W(hInst32);
}


/*
WOWGetProcAddress32 - This function calls thru to the Kernel call.

Returns - The 32 bit pointer to the function's entry point.
*/
FARPROC __export WINAPI WOWGetProcAddress32(HINSTANCE32 hInst32,
                                                           LPCSTR lpszProc)
{
    if(!(dwCapBits & WOW_GETPROCADDRESS))
        return NULL;
    
    return GetProcAddress32W(hInst32, lpszProc);
}


/*
WOWGetVDMPointer32 - This function calls thru to the Kernel call.

Returns - The 32 bit linear address of lpAddress.
*/
DWORD __export WINAPI WOWGetVDMPointer32(LPVOID lpAddress, UINT fMode)
{
    if(!(dwCapBits & WOW_VDMPTR32))
        return NULL;
    
    return GetVDMPointer32W(lpAddress, fMode);
}


/*
WOWCallProc32 - This function sets up a variable number of parameters and
                calls CallProc32W, which calls thru to the Win32 subsystem.

Returns - Whatever the called function is supposed to return.
*/
DWORD __export FAR CDECL WOWCallProc32(FARPROC lpfnFunction,
                             DWORD dwAddressConvert, DWORD dwParams, ...)
{
    va_list vaList;
    DWORD   dwCount;
    DWORD   dwTemp;
        
    
    if(!(dwCapBits & WOW_CALLPROC))
        return NULL;
        
    // Variable list start
    va_start(vaList,dwParams);
    
    for(dwCount=0; dwCount < dwParams; dwCount++)
    {
        // Pull each variable off of the stack
        dwTemp=(DWORD)va_arg(vaList,DWORD);
        
        // Push the DWORD
        __asm push word ptr [dwTemp+2];
        __asm push word ptr [dwTemp];
    }
    
    // Variable list end
    va_end(vaList);
    
    // Call Win32.  The pushed variable list precedes the parameters.
    // Appropriate parameters will be popped by this function (based
    // on the value in dwParams)
    return CallProc32W(lpfnFunction, dwAddressConvert, dwParams);
}


/*
WEP - Need I explain?
*/
int __export WINAPI WEP(int t)
{
    return 1;
}
