/*
  WOWTST16.C - 1994 James M. Finnegan
*/
#include <windows.h>
#include <dos.h>
#include "WOWGLUE.H"
#include "WOWTST16.H"
               
               
HINSTANCE hInst;

// Win32 instance (module) handles for WOWTST32 and KERNEL32 DLL's
HINSTANCE32 hInstWOW32;
HINSTANCE32 hInstKrnl32;

// Pointer to alias the GetCurrentTask function (to get at the returned DWORD)
DWORD (WINAPI *GetTask)(VOID);


int PASCAL WinMain(HINSTANCE hInstance, HANDLE hPrevInstance,
                                            LPSTR lpszCmdLine, int nCmdShow)
{
    static char szAppName[]="WOWTST16";
    HWND        hWnd;
    MSG         msg;
    WNDCLASS    wndclass;
    
    
    // Check to see if we are running under NT.  If we are not, bail...
    if(!(GetWinFlags() & WF_WINNT))
    {    
        MessageBox(NULL,"Hey! You are not running Windows NT!","Error",MB_OK);
        return -1;
    }

    // Make hInstance global
    hInst=hInstance;
    
    if(!hPrevInstance)
    {
        wndclass.style         = CS_HREDRAW | CS_VREDRAW;
        wndclass.lpfnWndProc   = Win16WndProc;
        wndclass.cbClsExtra    = 0;
        wndclass.cbWndExtra    = DLGWINDOWEXTRA;
        wndclass.hInstance     = hInstance;
        wndclass.hIcon         = LoadIcon(hInstance,szAppName);
        wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
        wndclass.hbrBackground = COLOR_WINDOW + 1;
        wndclass.lpszMenuName  = NULL;
        wndclass.lpszClassName = szAppName;

        if(!RegisterClass(&wndclass))
            return -1;
    }

    // Create the main window
    if((hWnd=CreateDialog(hInstance,MAKEINTRESOURCE(IDD_MAIN),0,NULL))==NULL)
        return -1;

    // Load the two referenced Win32 DLL's
    hInstWOW32 =WOWLoadLibraryEx32("WOWTST32.DLL",NULL,NULL);
    hInstKrnl32=WOWLoadLibraryEx32("KERNEL32.DLL",NULL,NULL);

    ShowWindow(hWnd,nCmdShow);
    UpdateWindow(hWnd);
    
    while(GetMessage(&msg,NULL,0,0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    // Free the loaded Win32 DLL's (this will not be done for us when
    // this thread ends!)
    WOWFreeLibrary32(hInstWOW32);
    WOWFreeLibrary32(hInstKrnl32);

    return msg.wParam;
}                 


/*
CenterWindow - Super duper window centering function
*/
VOID CenterWindow(HWND hWnd)
{
    RECT rect;
    WORD wWidth,
         wHeight;
         
         
    GetWindowRect(hWnd,&rect);

    wWidth =GetSystemMetrics(SM_CXSCREEN);
    wHeight=GetSystemMetrics(SM_CYSCREEN);

    MoveWindow(hWnd,(wWidth/2)   - ((rect.right -  rect.left)/2),
                    (wHeight/2)  - ((rect.bottom - rect.top) /2),
                     rect.right  -   rect.left,
                     rect.bottom -   rect.top, 
                     FALSE);
}


/*
Win16WndProc - Window procedure for the main window
*/
LONG __export WINAPI Win16WndProc(HWND hWnd,UINT uMessage,WPARAM wParam,
                                                            LPARAM lParam)
{
    // Function pointer for the WOWTST32 Win32CreateDialog function
    FARPROC lpfnCreateDialog;


    switch(uMessage)
    {
        case WM_CREATE:
            CenterWindow(hWnd);
            break;
            
        case WM_PAINT:
            // Update the static fields in the main window
            Win16UpdateStats(hWnd);
            DefWindowProc(hWnd,uMessage,wParam,lParam);
            break;
            
        case WM_DESTROY:
            PostQuitMessage(0);
            break;
        
        case WM_COMMAND:
            switch(wParam)
            {
                // Display modal dialog box of current Win16 tasks and their
                // Windows NT thread ID's
                case ID_TEST_LISTACTIVETHREADS:
                    DialogBox(hInst,MAKEINTRESOURCE(IDD_TASKLISTDIALOG),hWnd,
                                                  (FARPROC)Win16TaskDlgProc);
                    break;
            
                // Call Win32TestFunction in WOWTST32.DLL with various
                // parameters
                case ID_TEST_CALLWIN32TESTFUNCTION:
                	Win16CallTestFunc(hWnd);
                	break;
                	
                // Modal dialog box to show how Win32 can muck with your
                // Win16 windows
                case ID_TEST_POSTMESSAGESFROMWIN32:
                    DialogBox(hInst,MAKEINTRESOURCE(IDD_POSTMESSAGEDIALOG),
                                          hWnd,(FARPROC)Win16PostMsgDlgProc);
                    break;
                    
                // Modeless dialog box that creates a Win32 thread
                case ID_TEST_CREATEWIN16DIALOG:
                    CreateDialog(hInst,MAKEINTRESOURCE(IDD_WIN16DIALOG),hWnd,
                                                (FARPROC)Win16ThreadDlgProc);
                    break;
                    
                // This calls a WOWTST32 function that creates a modeless
                // dialog box from WOWTST32's resources.  The dialog
                // procedure is also included in WOWTST32, which creates
                // a Win32 thread. (All windows are created equal!)
                case ID_TEST_CREATEWIN32DIALOG:
                    if(lpfnCreateDialog=WOWGetProcAddress32(hInstWOW32,
                                                       "Win32CreateDialog"))
                        WOWCallProc32(lpfnCreateDialog,0,0);
                    break;
                    
                case ID_FILE_EXIT:
                    PostMessage(hWnd,WM_CLOSE,0,0L);
                    break;
                    
                default:
                    break;
            }
        
        default:
            return DefWindowProc(hWnd,uMessage,wParam,lParam);
            break;
          
    }
    return 0L;
}                                                            


/*
Win16UpdateStats - Calls the Win32 Kernel GetCurrentProcessId and
GetCurrentThreadId and displays the results.
*/
VOID Win16UpdateStats(HWND hWnd)
{
    FARPROC     lpfnWOW32Func;
    char        szText[80];
    

    if(lpfnWOW32Func=WOWGetProcAddress32(hInstKrnl32,"GetCurrentProcessId"))
    {
        wsprintf(szText,"%08lx",WOWCallProc32(lpfnWOW32Func,0,0));
        SetDlgItemText(hWnd,IDC_PROCESSID,szText);
    }

    if(lpfnWOW32Func=WOWGetProcAddress32(hInstKrnl32,"GetCurrentThreadId"))
    {
        wsprintf(szText,"%08lx",WOWCallProc32(lpfnWOW32Func,0,0));
        SetDlgItemText(hWnd,IDC_THREADID,szText);
    }
}


/*
Win16CallTestFunc - Exercises parameter conversion and passing to a Win32 
function
*/
VOID Win16CallTestFunc(HWND hWnd)
{
    // Four strings to pass to Win32TestFunction
    static char szText1[]="Parameter 1",
                szText2[]="Parameter 2",
                szText3[]="Parameter 3",
                szText4[]="Parameter 4";
    FARPROC     lpfnWin32Test;
    DWORD       dwLinAddr1,
                dwLinAddr2;
    
    
    if(lpfnWin32Test=WOWGetProcAddress32(hInstWOW32,"Win32TestFunction"))
    {
        // Fix our DS, since we will be getting the linear address, and we
        // don't want it to change while we are using it!
        GlobalFix(hInst);
        
        // Convert the selector:offset of these two strings to a linear
        // address for use by Win32
        dwLinAddr1=WOWGetVDMPointer32((LPSTR)szText1,1);
        dwLinAddr2=WOWGetVDMPointer32((LPSTR)szText2,1);

        // Call Win32TestFunction, converting the hWnd to 32 bits, and
        // passing the last two parameters as selector:offset, to be
        // converted by Win32
        WOWCallProc32(lpfnWin32Test,PARAM_01|PARAM_02,5,WOWHwndToHwnd32(hWnd),
                                                        dwLinAddr1,
                                                        dwLinAddr2,
                                                       (LPSTR)szText3,
                                                       (LPSTR)szText4);
        // Unfix our DS
        GlobalUnfix(hInst);
    }
}


/*
Win16TaskDlgProc - Dialog procedure for the Task and Thread ID list.
IDC_REFRESH walks the Win16 task list, displaying the module names and
the newly added Thread ID field.
*/
LONG __export WINAPI Win16TaskDlgProc(HWND hWnd,UINT uMessage,WPARAM wParam,
                                                              LPARAM lParam)
{
    static int iTabStop[]={32,95};
    WORD       wTask;
    LPSTR      lpszModName;
    DWORD FAR  *dwThreadId;
    char       szString[100];
    
                           
    switch(uMessage)
    {
        case WM_INITDIALOG:
            CenterWindow(hWnd);
            // Set the listbox's tab stops
            SendDlgItemMessage(hWnd,IDC_TASKLIST,LB_SETTABSTOPS,2,
                                                     (LONG)(LPINT)iTabStop);
            PostMessage(hWnd,WM_COMMAND,IDC_REFRESH,0L);
            break;
            
        case WM_CLOSE:
            PostMessage(hWnd,WM_COMMAND,IDCANCEL,0L);
            break;
            
        case WM_COMMAND:
            switch(wParam)
            {
                case IDC_REFRESH:
                    SendDlgItemMessage(hWnd,IDC_TASKLIST,LB_RESETCONTENT,0,0L);
                    
                    // Alias GetCurrentTask, so we can get at the DWORD
                    (FARPROC)GetTask=(FARPROC)GetCurrentTask;
                    // Get the HIWORD returned by GetCurrentTask.  This 
                    // value is the head of the Task Data Block (TDB)
                    wTask=HIWORD(GetTask());

                    // Walk the task list
                    while(wTask)
                    {
                        // The module name is at offset 0xf2
                        lpszModName=MK_FP(wTask,0xf2);
                        // The Thread ID field is at offset 0xfc
                        dwThreadId =MK_FP(wTask,0xfc);
                        
                        // Print 'em!
                        wsprintf(szString,"%04x\t%-8.8s\t%08lx",wTask,
                                                                lpszModName,
                                                                *dwThreadId);
                                                                    
                        SendDlgItemMessage(hWnd,IDC_TASKLIST,LB_ADDSTRING,0,
                                                  (LONG)(LPSTR)szString);
                        
                        // Get the next TDB (0 if we're at the end)
                        wTask=*(HANDLE FAR *)MK_FP(wTask,0);
                    }         
                    break;

                case IDCANCEL:
                    EndDialog(hWnd,FALSE);
                    break;
            }
            break;
            
        default:
            return FALSE;
    }
    return TRUE;
}
             
             
/*
Win16PostMsgDlgProc - Dialog procedure that demonstrates how Win32 can
"talk" to you
*/
LONG __export WINAPI Win16PostMsgDlgProc(HWND hWnd,UINT uMessage,
                                                WPARAM wParam,LPARAM lParam)
{
    FARPROC lpfnPostWOWMessages; // function pointer for Win32PostWOWMessages
    char    szText[80];          // Scratch text buffer
    UINT    uSel;                // Selector for aliasing a linear address
                          
                          
    switch(uMessage)
    {
        case WM_INITDIALOG:
            CenterWindow(hWnd);
            break;

        // We are intercepting WM_SETTEXT to show that standard Windows
        // message parameters are automatically converted to 16 bits
        // for us
        case WM_SETTEXT:
            // Note we are using lParam without converting it!
            MessageBox(hWnd,(LPSTR)lParam,"Win16 WM_SETTEXT Message",MB_OK);
            // Pass the message back to Windows to change the title
            DefWindowProc(hWnd,uMessage,wParam,lParam);
            break;
            
        // This message is sent to us by Win32PostWOWMessages in WOWTST32.DLL.
        // User defined messages DO NOT have their parameters converted!
        case WM_USER+100:
            // Create a selector with a linear base address of lParam (this
            // is the linear address of the string passed by the WOWTST32
            // PostMessage call).  Be sure not to specify a limit past the end
            // of WOWTST32's data segment, or else bad things will happen.
            uSel=WOWCreateVDMPointer16(lParam,50);
            
            // Print wParam, lParam, and the string passed by lParam (aliased
            // by uSel)
            wsprintf(szText,"wParam == %x  lParam == %08lx\n\nAliased lParam* == '%s'",
                                                      wParam,lParam,
                                                      (LPSTR)MK_FP(uSel,0));
            MessageBox(hWnd,szText,"Win16 WM_USER+100 Message",MB_OK);
  
            // Delete the previously created data selector
            WOWDeleteVDMPointer16(uSel);                                
            break;
                
        case WM_CLOSE:
            PostMessage(hWnd,WM_COMMAND,IDCANCEL,0L);
            break;
            
        case WM_COMMAND:
            lpfnPostWOWMessages=WOWGetProcAddress32(hInstWOW32,
                                                    "Win32PostWOWMessages");

            // Call Win32PostWOWMessages with our hWnd (converted to 32 bits
            // by WOWHwndToHwnd32).  This will result in messages being sent
            // back to us via SendMessage or PostMessage from Win32.
            switch(wParam)
            {
                // Win32PostWOWMessages will send us a WM_GETTEXT message and
                // display the result in a Win32 message box
                case IDC_GETTEXT:
                    WOWCallProc32(lpfnPostWOWMessages,0,2,
                                            (HWND32)WOWHwndToHwnd32(hWnd),
                                            (DWORD)0 );
                    break;

                // Win32PostWOWMessages will send us a WM_SETTEXT message.
                // Note that we are intercepting this message above
                case IDC_SETTEXT:
                    WOWCallProc32(lpfnPostWOWMessages,0,2,
                                            (HWND32)WOWHwndToHwnd32(hWnd),
                                            (DWORD)1 );
                    break;

                // Win32PostWOWMessages will send us a WM_USER+100 message.
                // This is handled above
                case IDC_USER:
                    WOWCallProc32(lpfnPostWOWMessages,0,2,
                                            (HWND32)WOWHwndToHwnd32(hWnd),
                                            (DWORD)2 );
                    break;

                case IDCANCEL:
                    EndDialog(hWnd,FALSE);
                    break;
            }
            break;
            
        default:
            return FALSE;
    }
    return TRUE;
}

/*
Win16ThreadDlgProc - Dialog Box procedure for our Win16 thread
*/
LONG __export WINAPI Win16ThreadDlgProc(HWND hWnd,UINT uMessage,
	                                            WPARAM wParam,LPARAM lParam)
{
    FARPROC lpfnCreateCountThread;
    DWORD   dwThreadId;
    DWORD   hThread;
    HWND32  hTextWnd32;
    char    szText[80];


    switch(uMessage)
    {
        case WM_INITDIALOG:
            if(lpfnCreateCountThread=WOWGetProcAddress32(hInstWOW32,
                                                  "Win32CreateCountThread"))
            {
                // Get static window and convert it into a Win32 HWND
                hTextWnd32=WOWHwndToHwnd32(GetDlgItem(hWnd,IDC_TEXT2));
            
                // Call CreateCountThread, which will create a thread
                // that will update the window specified in hTextWnd32
                if(hThread=WOWCallProc32(lpfnCreateCountThread,
                                                  PARAM_01,2,
                                                  (HWND32)hTextWnd32,
                                                  (DWORD FAR *)&dwThreadId))
                {
                    // Print the new thread's thread handle and ID
                    wsprintf(szText,"WOW Thread == %08lx  ID == %08lx",
                                                     hThread,dwThreadId);
                    SetDlgItemText(hWnd,IDC_TEXT1,szText);
                }
                else
                    MessageBox(hWnd,"Thread Create Failed!","Oh No!",MB_OK);
            }    
            break;

        case WM_CLOSE:
            PostMessage(hWnd,WM_COMMAND,IDCANCEL,0);
            break;

        case WM_COMMAND:
            switch(wParam)
            {
                case IDCANCEL:
                    DestroyWindow(hWnd);
                    break;
            }
            break;

        default:
            return FALSE;
    }
    return TRUE;
}
