/*
  WOWTST32.C - 1994 James M. Finnegan
*/
#include <windows.h>
#include <winbase.h>
#include "WOWTST32.H"

// hInst global for Win32 CreateDialog()
HINSTANCE hInst;


/*
DllMain - This is the entry point for a Win32 DLL.  It is called whenever
a process or thread attaches or detaches from it.  My implementation calls
user interface code (MessageBox) to demonstrate the order of events when
processes and threads use this DLL.  User interface code in this function
probably isn't a good idea, since NT doesn't seem to deal with it well under
extraordinary conditions.

Returns - Nonzero if life is good.
*/
BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID LPReserved)
{
    LPSTR lpszReason;
    char  szString[100];


    switch(fdwReason)
    {
        case DLL_PROCESS_ATTACH:
            hInst=hInstDLL;
            lpszReason="Attach by Process";
            break;

        case DLL_THREAD_ATTACH:
            lpszReason="Attach by Thread";
            break;

        case DLL_PROCESS_DETACH:
            lpszReason="Detach by Process";
            break;

        case DLL_THREAD_DETACH:
            lpszReason="Detach by Thread";
            break;

        default:
            lpszReason="Who knows?!";
            break;
    }

    wsprintf(szString,"%s\n\nProcess: %08lx\nThread: %08lx",
                                               lpszReason,
                                               GetCurrentProcessId(),
                                               GetCurrentThreadId());
    // This is system modal so it always appears on the top of other 
    // windows.  As you'll see, system modal is not system modal!
    MessageBox(NULL,szString,"WOW 32 Test DLL",MB_OK|MB_SYSTEMMODAL);

    return 1;
}

/*
Win32TestFunction - This function simply prints the parameters passed to
it in a message box.  hWnd is the parent window handle for MessageBox.
This function is used to exercise parameter passing from a Win16
application.
*/
VOID WINAPI Win32TestFunction(HWND hWnd,LPSTR lpszParam1, LPSTR lpszParam2,
                                        LPSTR lpszParam3, LPSTR lpszParam4)
{
    char szString[100];


    wsprintf(szString,"Called by:\nPID %lx\nTID %lx\n\n%s\n%s\n%s\n%s",
                                                     GetCurrentProcessId(),
                                                     GetCurrentThreadId(),
                                                     lpszParam1,lpszParam2,
                                                     lpszParam3,lpszParam4);
    
    MessageBox(hWnd,szString,"Win32 Test DLL",MB_OK);
}


/*
Win32PostWOWMessages - This function sends and posts messages to the window
specified in hWnd.  This is used to demonstrate message passing to a Win16
app from Win32.
*/
VOID WINAPI Win32PostWOWMessages(HWND hWnd,DWORD dwFunction)
{
    char szText[80];
    static DWORD i=1;


    switch(dwFunction)
    {
        case 0:
            // Send a WM_GETTEXT to hWnd and print the result
            SendMessage(hWnd,WM_GETTEXT,sizeof(szText),(LONG)(LPSTR)szText);
            MessageBox(hWnd,szText,"Result of Win32 GetWindowText",MB_OK);
            break;

        case 1:
            // Send a WM_SETTEXT to hWnd
            wsprintf(szText,"New Title #%ld from Win32",i++);
            SendMessage(hWnd,WM_SETTEXT,0,(LONG)(LPSTR)szText);
            break;
        
        case 2:
            // Post a user defined message with one LONG parameter and one
            // linear address parameter
            PostMessage(hWnd,WM_USER+100,0x12345678,
                                            (LONG)(LPSTR)"Text from Win32");
            break;
    }
}


/*
Win32CreateDialog - This function creates a modeless dialog box using
the resource script and callback function of WOWTST32.DLL (this file).
The callback function ultimately creates a thread that periodically 
updates a static text window in the dialog. Used to demonstrate creating
a window and thread in Win32 code when called from a Win16 application.  

Returns - Whatever CreateDialog returns.
*/
HWND WINAPI Win32CreateDialog()
{
    // Use the hInst of the DLL to load the dialog resource.
    return CreateDialog(hInst,MAKEINTRESOURCE(IDD_DIALOG),GetActiveWindow(),
                                                              Win32DlgProc);
}


/*
Win32DlgProc - This is the dialog box procedure for the dialog created above
*/
LONG WINAPI Win32DlgProc(HWND hWnd,UINT uMessage,WPARAM wParam,LPARAM lParam)
{
    DWORD  dwThreadId;
    HANDLE hThread;
    HWND   hTextWnd;
    char   szText[80];


    switch(uMessage)
    {
        case WM_INITDIALOG:
            // Get the static window handle
            hTextWnd=GetDlgItem(hWnd,IDC_TEXT2);
    
            // Create the thread
            if(hThread=Win32CreateCountThread(hTextWnd,&dwThreadId))
            {
                // Format and print the newly created thread handle and ID
                wsprintf(szText,"WOW Thread == %08lx  ID == %08lx",hThread,
                                                                 dwThreadId);
                SetDlgItemText(hWnd,IDC_TEXT1,szText);
            }
            else
                 MessageBox(hWnd,"Thread Create Failed!","Oh No!",MB_OK);
            break;

        case WM_CLOSE:
            PostMessage(hWnd,WM_COMMAND,IDCANCEL,0);
            break;

        case WM_COMMAND:
            switch(wParam)
            {
                case IDCANCEL:
                    DestroyWindow(hWnd);
                    break;
            }
            break;

        default:
            return FALSE;
    }
    return TRUE;

}
                                       
                                       
/*
Win32CreateCountThread - This function creates a thread in the current
process space with the Win32ThreadLoop thread function.  The hWnd parameter
indicates the window that Win32ThreadLoop will modify.  dwThreadId is a 
pointer to a long int that will be updated with the newly created thread ID.
This function is called from both Win32DlgProc (above) and directly from
WOWTST16.EXE.  It demonstrates creating threads in NTVDM, as well as showing
how a Win32 thread can interact with a Win16 window.

Returns - The newly created thread handle.
*/
HANDLE WINAPI Win32CreateCountThread(HWND hWnd, DWORD *dwThreadId)
{
    HANDLE hThread;


    // Create a new thread, passing hWnd as a parameter to Win32ThreadLoop
    if(!(hThread=CreateThread(NULL,0,Win32ThreadLoop,hWnd,0,dwThreadId)))
        return NULL;
    
    // Bump the new thread's priority down one notch, so as not to compete
    // with the parent thread
    SetThreadPriority(hThread,THREAD_PRIORITY_BELOW_NORMAL);

    return hThread;
}


/*
Win32ThreadLoop - This is the function used by CreateThread (above) to 
infinitely loop while updating hWnd's text with the current iteration.
This ends when hWnd becomes invalid (when it is destroyed).  Since this
function never "sleeps", it chews up gobs of CPU time (bad, bad!)

Returns - 0 when the thread exits.
*/
DWORD WINAPI Win32ThreadLoop(HWND hWnd)
{
    DWORD i=0;
    char  szString[40];

    // Loop until hWnd goes away
    while(IsWindow(hWnd))
    {
        wsprintf(szString,"Pass %ld",i++);
        SetWindowText(hWnd,szString);
    }

    // Ahhh, breathe a sigh of relief...
    MessageBeep(0);
    return 0;
}
